const config = require('config')
const { get } = require('lodash')

const {
  User,
  Email,
  services,
  Fragment,
} = require('pubsweet-component-helper-service')

const resetPath = config.get('invite-reset-password.url')
const journalName = config.get('journal.name')
const { getEmailCopy } = require('./emailCopy')

const unsubscribeSlug = config.get('unsubscribe.url')
const editorialAssistantEmail = config.get('journal.staffEmail')

module.exports = {
  async sendNotifications({
    baseUrl,
    fragment,
    UserModel,
    collection,
    isNewVersion = false,
    isTechnicalCheck,
    isMajorRecommendation,
  }) {
    const fragmentHelper = new Fragment({ fragment })
    const parsedFragment = await fragmentHelper.getFragmentData({
      handlingEditor: collection.handlingEditor,
    })
    const {
      submittingAuthor,
      activeAuthors: authors,
    } = await fragmentHelper.getAuthorData({
      UserModel,
    })

    const { customId } = collection
    const titleText = `the manuscript titled "${parsedFragment.title}" by ${
      submittingAuthor.firstName
    } ${submittingAuthor.lastName}`

    const email = new Email({
      type: 'user',
      content: {
        signatureName: '',
        ctaLink: services.createUrl(
          baseUrl,
          `/projects/${collection.id}/versions/${fragment.id}/details`,
        ),
        ctaText: 'MANUSCRIPT DETAILS',
      },
    })

    const userHelper = new User({ UserModel })
    const eicName = await userHelper.getEiCName()
    if (isNewVersion) {
      sendHandlingEditorEmail({
        email,
        eicName,
        baseUrl,
        customId,
        title: parsedFragment.title,
        handlingEditor: get(collection, 'handlingEditor', {}),
      })
    }

    if (isMajorRecommendation) {
      sendReviewersEmail({
        email,
        baseUrl,
        customId,
        UserModel,
        fragmentHelper,
        title: parsedFragment.title,
      })
    }

    if (isTechnicalCheck) {
      sendEQSEmail({
        email,
        eicName,
        baseUrl,
        collection,
      })
      sendAuthorsEmail({
        email,
        baseUrl,
        titleText,
        UserModel,
        fragmentId: fragment.id,
        fragmentAuthors: authors,
        collectionId: collection.id,
      })
    }
  },
}

const sendHandlingEditorEmail = ({
  email,
  title,
  eicName,
  baseUrl,
  customId,
  handlingEditor,
}) => {
  const emailType = 'he-new-version-submitted'

  email.toUser = {
    email: handlingEditor.email,
    name: handlingEditor.name,
  }
  email.content.unsubscribeLink = services.createUrl(baseUrl, unsubscribeSlug, {
    id: handlingEditor.id,
  })
  email.content.signatureName = eicName
  email.content.subject = `${customId}: Revision submitted`

  const { html, text } = email.getBody({
    body: getEmailCopy({
      emailType,
      titleText: `the manuscript titled ${title}`,
    }),
  })
  email.sendEmail({ html, text })
}

const sendReviewersEmail = async ({
  email,
  title,
  baseUrl,
  customId,
  UserModel,
  fragmentHelper,
}) => {
  email.content.subject = `${customId}: A manuscript you reviewed has been revised`
  const emailType = 'submitted-reviewers-after-revision'

  const reviewers = await fragmentHelper.getReviewers({
    UserModel,
    type: 'submitted',
  })

  reviewers.forEach(reviewer => {
    email.toUser = {
      email: reviewer.email,
      name: `${reviewer.lastName}`,
    }

    email.content.unsubscribeLink = services.createUrl(
      baseUrl,
      unsubscribeSlug,
      {
        id: reviewer.id,
      },
    )

    const { html, text } = email.getBody({
      body: getEmailCopy({
        emailType,
        titleText: `the manuscript titled ${title}`,
        expectedDate: services.getExpectedDate({ daysExpected: 14 }),
      }),
    })

    email.sendEmail({ html, text })
  })
}

const sendEQSEmail = ({
  email,
  eicName,
  baseUrl,
  collection,
  subjectBaseText,
}) => {
  const emailType = 'eqs-manuscript-submitted'

  email.toUser = {
    email: editorialAssistantEmail,
    name: 'Editorial Assistant',
  }

  email.content.unsubscribeLink = baseUrl
  email.content.signatureName = eicName
  email.content.subject = `${subjectBaseText} Submitted`
  email.content.ctaLink = services.createUrl(
    baseUrl,
    config.get('eqs-decision.url'),
    {
      collectionId: collection.id,
      customId: collection.customId,
      token: collection.technicalChecks.token,
    },
  )
  email.content.ctaText = 'MAKE DECISION'

  const { html, text } = email.getBody({
    body: getEmailCopy({
      emailType,
      customId: collection.customId,
    }),
  })
  email.sendEmail({ html, text })
}

const sendAuthorsEmail = async ({
  email,
  title,
  baseUrl,
  customId,
  UserModel,
  titleText,
  fragmentId,
  collectionId,
  fragmentAuthors,
}) => {
  email.content.subject = `Manuscript submitted to ${journalName}`

  const userEmailData = await Promise.all(
    fragmentAuthors.map(async author => {
      const user = await UserModel.find(author.id)
      return {
        ...author,
        isConfirmed: user.isConfirmed,
        ...getEmailCopy({
          customId,
          emailType: author.isSubmitting
            ? 'submitting-author-manuscript-submitted'
            : 'coauthors-manuscript-submitted',
          titleText: author.isSubmitting
            ? titleText
            : `The manuscript titled "${title}" has been submitted to ${journalName} by ${
                author.firstName
              } ${author.lastName}`,
        }),
      }
    }),
  )

  userEmailData.forEach(author => {
    email.toUser = {
      email: author.email,
      name: `${author.lastName}`,
    }
    email.content.unsubscribeLink = services.createUrl(
      baseUrl,
      unsubscribeSlug,
      {
        id: author.id,
      },
    )

    if (author.isSubmitting) {
      email.content.ctaLink = services.createUrl(
        baseUrl,
        `/projects/${collectionId}/versions/${fragmentId}/details`,
      )
      email.content.ctaText = 'MANUSCRIPT DETAILS'
    } else if (author.isConfirmed) {
      email.content.ctaLink = services.createUrl(baseUrl, '')
      email.content.ctaText = 'LOGIN'
    } else {
      email.content.ctaLink = services.createUrl(baseUrl, resetPath, {
        email: author.email,
        token: author.passwordResetToken,
        firstName: author.firstName,
        lastName: author.lastName,
        affiliation: author.affiliation,
        title: author.title,
      })
      email.content.ctaText = 'CONFIRM ACCOUNT'
    }

    const { html, text } = email.getBody({
      body: {
        paragraph: author.paragraph,
        hasLink: author.hasLink,
      },
    })
    email.sendEmail({ html, text })
  })
}
